@@ -1,5 +1,5 @@
 import subprocess
-from ejcli.http import task_list, task_ids, download_file
+from ejcli.http import tasks, download_file
 from ejcli.error import EJError
 
 def do_getfile(self, cmd):
@@ -9,11 +9,10 @@ def do_getfile(self, cmd):
     Retrieve a file attached to a task.
     """
     id, file, cmd = (cmd.strip()+'  ').split(' ', 2)
-    tasks = task_list(self.url, self.cookie)
     if not id or not file:
         return self.do_help('getfile')
-    try: task_id = task_ids(self.url, self.cookie)[tasks.index(id)]
-    except (ValueError, IndexError):
+    try: task_id = next(i.id for i in tasks(self.url, self.cookie) if i.short_name == id)
+    except StopIteration:
         raise EJError("No such task.")
     data = download_file(self.url, self.cookie, task_id, file)
     p = subprocess.Popen('cat '+cmd, stdin=subprocess.PIPE, shell=True)
